import rdflib
from rdflib import RDF, Graph, Literal


def testPythonRoundtrip():
    l1 = Literal('<msg>hello</msg>', datatype=RDF.XMLLiteral)
    assert l1.value is not None, 'xml must have been parsed'
    assert l1.datatype == RDF.XMLLiteral, 'literal must have right datatype'

    l2 = Literal('<msg>good morning</msg>', datatype=RDF.XMLLiteral)
    assert l2.value is not None, 'xml must have been parsed'
    assert not l1.eq(l2), 'literals must NOT be equal'

    l3 = Literal(l1.value)
    assert l1.eq(l3), 'roundtripped literals must be equal'
    assert l3.datatype == RDF.XMLLiteral, 'literal must have right datatype'

    l4 = Literal('<msg >hello</msg>', datatype=RDF.XMLLiteral)
    assert l1 == l4
    assert l1.eq(l4)

    rdflib.NORMALIZE_LITERALS = False
    l4 = Literal('<msg >hello</msg>', datatype=RDF.XMLLiteral)
    assert l1 != l4
    assert l1.eq(l4)
    rdflib.NORMALIZE_LITERALS = True


def testRDFXMLParse():
    rdfxml = """\
<rdf:RDF
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
>

<rdf:Description rdf:about="http://example.org/">
    <dc:description rdf:parseType="Literal">
        <p xmlns="http://www.w3.org/1999/xhtml" xml:lang="en"></p>
    </dc:description>
</rdf:Description>

</rdf:RDF>"""

    g = rdflib.Graph()
    g.parse(data=rdfxml)
    l1 = list(g)[0][2]
    assert l1.datatype == RDF.XMLLiteral


def graph():
    g = rdflib.Graph()
    g.add((rdflib.URIRef('http://example.org/a'),
           rdflib.URIRef('http://example.org/p'),
           rdflib.Literal('<msg>hei</hei>', datatype=RDF.XMLLiteral)))
    return g


def roundtrip(fmt):
    g1 = graph()
    l1 = list(g1)[0][2]
    g2 = rdflib.Graph()
    g2.parse(data=g1.serialize(format=fmt), format=fmt)
    l2 = list(g2)[0][2]
    assert l1.eq(l2)


def testRoundtrip():
    roundtrip('xml')
    roundtrip('n3')
    roundtrip('nt')


def testHTML():

    l1 = Literal('<msg>hello</msg>', datatype=RDF.XMLLiteral)
    assert l1.value is not None, 'xml must have been parsed'
    assert l1.datatype == RDF.XMLLiteral, 'literal must have right datatype'

    l2 = Literal('<msg>hello</msg>', datatype=RDF.HTML)
    assert l2.value is not None, 'xml must have been parsed'
    assert l2.datatype == RDF.HTML, 'literal must have right datatype'

    assert l1 != l2
    assert not l1.eq(l2)



"""

RDFLib Graphs (and Resources) can be "sliced" with [] syntax

This is a short-hand for iterating over triples

Combined with SPARQL paths (see ``foafpaths.py``) - quite complex queries
can be realised.

See :meth:`rdflib.graph.Graph.__getitem__` for details

"""

from rdflib import Graph, RDF
from rdflib.namespace import FOAF

if __name__ == '__main__':

    graph = Graph()

    graph.load("foaf.rdf")

    for person in graph[: RDF.type: FOAF.Person]:

        friends = list(graph[person:FOAF.knows * '+' / FOAF.name])
        if friends:
            print("%s's circle of friends:" % graph.value(person, FOAF.name))
            for name in friends:
                print(name)



import unittest
import rdflib
from rdflib.compare import graph_diff

"""Test for graph_diff - much more extensive testing 
would certainly be possible"""


class TestDiff(unittest.TestCase):
    """Unicode literals for graph_diff test
    (issue 151)"""

    def testA(self):
        """with bnode"""
        g = rdflib.Graph()
        g.add((rdflib.BNode(), rdflib.URIRef("urn:p"), rdflib.Literal(u'\xe9')))

        diff = graph_diff(g, g)

    def testB(self):
        """Curiously, this one passes, even before the fix in issue 151"""

        g = rdflib.Graph()
        g.add((rdflib.URIRef("urn:a"), rdflib.URIRef("urn:p"), rdflib.Literal(u'\xe9')))

        diff = graph_diff(g, g)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python


from rdflib.graph import ConjunctiveGraph
import unittest


class TestTrixParse(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testAperture(self):

        g = ConjunctiveGraph()

        g.parse("test/trix/aperture.trix", format="trix")
        c = list(g.contexts())

        # print list(g.contexts())
        t = sum(map(len, g.contexts()))

        self.assertEqual(t, 24)
        self.assertEqual(len(c), 4)

        # print "Parsed %d triples"%t

    def testSpec(self):

        g = ConjunctiveGraph()

        g.parse("test/trix/nokia_example.trix", format="trix")

        # print "Parsed %d triples"%len(g)

    def testNG4j(self):

        g = ConjunctiveGraph()

        g.parse("test/trix/ng4jtest.trix", format="trix")

        # print "Parsed %d triples"%len(g)


if __name__ == '__main__':
    unittest.main()



"""Runs the tests for the W3C RDF Working Group's TriG test suite.

"""

from rdflib import ConjunctiveGraph
from rdflib.namespace import split_uri
from rdflib.compare import graph_diff, isomorphic

from .manifest import nose_tests, RDFT
from .testutils import nose_tst_earl_report

verbose = False


def trig(test):
    g = ConjunctiveGraph()

    try:
        base = 'http://www.w3.org/2013/TriGTests/' + split_uri(test.action)[1]

        g.parse(test.action, publicID=base, format='trig')
        if not test.syntax:
            raise AssertionError("Input shouldn't have parsed!")

        if test.result:  # eval test
            res = ConjunctiveGraph()
            res.parse(test.result, format='nquads')

            if verbose:

                both, first, second = graph_diff(g, res)
                if not first and not second:
                    return

                print('===============================')
                print('TriG')
                print(g.serialize(format='nquads'))
                print('===============================')
                print('NQuads')
                print(res.serialize(format='nquads'))
                print('===============================')

                print("Diff:")
                # print "%d triples in both"%len(both)
                print("TriG Only:")
                for t in first:
                    print(t)

                print("--------------------")
                print("NQuads Only")
                for t in second:
                    print(t)
                raise Exception('Graphs do not match!')

            assert isomorphic(g, res), 'graphs must be the same'

    except:
        if test.syntax:
            raise


testers = {
    RDFT.TestTrigPositiveSyntax: trig,
    RDFT.TestTrigNegativeSyntax: trig,
    RDFT.TestTrigEval: trig,
    RDFT.TestTrigNegativeEval: trig
}


def test_trig(tests=None):
    for t in nose_tests(testers, 'test/w3c/trig/manifest.ttl'):
        if tests:
            for test in tests:
                if test in t[1].uri:
                    break
            else:
                continue

        yield t


if __name__ == '__main__':
    verbose = True

    nose_tst_earl_report(test_trig, 'rdflib_trig')



"""
some more specific Literal tests are in test_literal.py
"""

import base64
import unittest
import random

from rdflib.term import URIRef, BNode, Literal, _is_valid_unicode
from rdflib.graph import QuotedGraph, Graph
from rdflib.namespace import XSD

from six import PY3


def uformat(s):
    if PY3:
        return s.replace("u'", "'")
    return s


class TestURIRefRepr(unittest.TestCase):
    """
    see also test_literal.TestRepr
    """

    def testSubclassNameAppearsInRepr(self):
        class MyURIRef(URIRef):
            pass
        x = MyURIRef('http://example.com/')
        self.assertEqual(repr(x), uformat("MyURIRef(u'http://example.com/')"))

    def testGracefulOrdering(self):
        u = URIRef('cake')
        g = Graph()
        a = u > u
        a = u > BNode()
        a = u > QuotedGraph(g.store, u)
        a = u > g


class TestBNodeRepr(unittest.TestCase):

    def testSubclassNameAppearsInRepr(self):
        class MyBNode(BNode):
            pass
        x = MyBNode()
        self.assertTrue(repr(x).startswith("MyBNode("))


class TestLiteral(unittest.TestCase):

    def test_base64_values(self):
        b64msg = 'cmRmbGliIGlzIGNvb2whIGFsc28gaGVyZSdzIHNvbWUgYmluYXJ5IAAR83UC'
        decoded_b64msg = base64.b64decode(b64msg)
        lit = Literal(b64msg, datatype=XSD.base64Binary)
        self.assertEqual(lit.value, decoded_b64msg)
        self.assertEqual(str(lit), b64msg)

    def test_total_order(self):
        types = {
            XSD.dateTime: (
                '2001-01-01T00:00:00',
                '2001-01-01T00:00:00Z',
                '2001-01-01T00:00:00-00:00'
            ),
            XSD.date: (
                '2001-01-01',
                '2001-01-01Z',
                '2001-01-01-00:00'
            ),
            XSD.time: (
                '00:00:00',
                '00:00:00Z',
                '00:00:00-00:00'
            ),
            XSD.gYear: (
                '2001',
                '2001Z',
                '2001-00:00'
            ),  # interval
            XSD.gYearMonth: (
                '2001-01',
                '2001-01Z',
                '2001-01-00:00'
            ),
        }
        literals = [
            Literal(literal, datatype=t)
            for t, literals in types.items()
            for literal in literals
        ]
        try:
            sorted(literals)
            orderable = True
        except TypeError as e:
            for l in literals:
                print(repr(l), repr(l.value))
            print(e)
            orderable = False
        self.assertTrue(orderable)

        # also make sure that within a datetime things are still ordered:
        l1 = [
            Literal(l, datatype=XSD.dateTime)
            for l in [
                '2001-01-01T00:00:00',
                '2001-01-01T01:00:00',
                '2001-01-01T01:00:01',
                '2001-01-02T01:00:01',
                '2001-01-01T00:00:00Z',
                '2001-01-01T00:00:00-00:00',
                '2001-01-01T01:00:00Z',
                '2001-01-01T01:00:00-00:00',
                '2001-01-01T00:00:00-01:30',
                '2001-01-01T01:00:00-01:30',
                '2001-01-02T01:00:01Z',
                '2001-01-02T01:00:01-00:00',
                '2001-01-02T01:00:01-01:30'
            ]
        ]
        l2 = list(l1)
        random.shuffle(l2)
        self.assertListEqual(l1, sorted(l2))


class TestValidityFunctions(unittest.TestCase):

    def test_is_valid_unicode(self):
        testcase_list = (
            (None, True),
            (1, True),
            (['foo'], True),
            ({'foo': b'bar'}, True),
            ('foo', True),
            (b'foo\x00', True),
            (b'foo\xf3\x02', False)
        )
        for val, expected in testcase_list:
            self.assertEqual(_is_valid_unicode(val), expected)



import os
import sys
import logging

log = logging.getLogger(__name__)

try:
    from .testutils import check_serialize_parse
except:
    from test.testutils import check_serialize_parse


def _get_test_files_formats():
    skiptests = [
    ]
    for f in os.listdir('test/n3'):
        if f not in skiptests:
            fpath = "test/n3/" + f
            if f.endswith('.rdf'):
                yield fpath, 'xml'
            elif f.endswith('.n3'):
                yield fpath, 'n3'

def all_n3_files():
    skiptests = [
        'test/n3/example-lots_of_graphs.n3',  # only n3 can serialize QuotedGraph, no point in testing roundtrip
    ]
    for fpath, fmt in _get_test_files_formats():
        if fpath in skiptests:
            log.debug("Skipping %s, known issue" % fpath)
        else:
            yield fpath, fmt

def test_n3_writing():
    for fpath, fmt in _get_test_files_formats():
        yield check_serialize_parse, fpath, fmt, 'n3'


if __name__ == "__main__":
    if len(sys.argv) > 1:
        check_serialize_parse(sys.argv[1], 'n3', 'n3', True)
        sys.exit()
    else:
        import nose
        nose.main(defaultTest=__name__)



from nose.exc import SkipTest
from nose.tools import nottest
import sys
import os
from tempfile import mkdtemp, mkstemp
from rdflib import RDF, RDFS, URIRef, BNode, Variable, plugin
from rdflib.graph import QuotedGraph, ConjunctiveGraph


implies = URIRef("http://www.w3.org/2000/10/swap/log#implies")
testN3 = """
@prefix rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
@prefix : <http://test/> .
{:a :b :c;a :foo} => {:a :d :c,?y}.
_:foo a rdfs:Class.
:a :d :c."""


# Thorough test suite for formula-aware store


@nottest  # do not run on its own - only as part of generator
def testFormulaStore(store="default", configString=None):
    try:
        g = ConjunctiveGraph(store=store)
    except ImportError:
        raise SkipTest("Dependencies for store '%s' not available!" % store)

    if configString:
        g.destroy(configString)
        g.open(configString)
    else:
        if store == 'SQLite':
            _, path = mkstemp(prefix='test', dir='/tmp', suffix='.sqlite')
            g.open(path, create=True)
        else:
            g.open(mkdtemp(), create=True)

    g.parse(data=testN3, format="n3")
    try:
        for s, p, o in g.triples((None, implies, None)):
            formulaA = s
            formulaB = o

        assert type(formulaA) == QuotedGraph and type(formulaB) == QuotedGraph
        # a = URIRef('http://test/a')
        b = URIRef('http://test/b')
        c = URIRef('http://test/c')
        d = URIRef('http://test/d')
        v = Variable('y')

        universe = ConjunctiveGraph(g.store)

        # test formula as terms
        assert len(list(universe.triples((formulaA, implies, formulaB)))) == 1

        # test variable as term and variable roundtrip
        assert len(list(formulaB.triples((None, None, v)))) == 1
        for s, p, o in formulaB.triples((None, d, None)):
            if o != c:
                assert isinstance(o, Variable)
                assert o == v
        s = list(universe.subjects(RDF.type, RDFS.Class))[0]
        assert isinstance(s, BNode)
        assert len(list(universe.triples((None, implies, None)))) == 1
        assert len(list(universe.triples((None, RDF.type, None)))) == 1
        assert len(list(formulaA.triples((None, RDF.type, None)))) == 1
        assert len(list(formulaA.triples((None, None, None)))) == 2
        assert len(list(formulaB.triples((None, None, None)))) == 2
        assert len(list(universe.triples((None, None, None)))) == 3
        assert len(list(formulaB.triples(
            (None, URIRef('http://test/d'), None)))) == 2
        assert len(list(universe.triples(
            (None, URIRef('http://test/d'), None)))) == 1

        # #context tests
        # #test contexts with triple argument
        # assert len(list(universe.contexts((a, d, c)))) == 1, \
        #                     [ct for ct in universe.contexts((a, d, c))]

        # FAIL: test.test_graph_formula.testFormulaStores('SQLite',)
        # --------------------------------------------------------------------
        # Traceback (most recent call last):
        #   File ".../site-packages/nose/case.py", line 197, in runTest
        #     self.test(*self.arg)
        #   File ".../test_graph_formula.py", line 80, in testFormulaStore
        #     [ct for ct in universe.contexts((a, d, c))]
        # AssertionError: [
        #     <Graph identifier=N52fd4417ef7641089b2e4045ef19ad87
        #        (<class 'rdflib.graph.Graph'>)>,
        #     <Graph identifier=_:Formula16 (<class 'rdflib.graph.Graph'>)>
        #     ]

        # Remove test cases
        universe.remove((None, implies, None))
        assert len(list(universe.triples((None, implies, None)))) == 0
        assert len(list(formulaA.triples((None, None, None)))) == 2
        assert len(list(formulaB.triples((None, None, None)))) == 2

        formulaA.remove((None, b, None))
        assert len(list(formulaA.triples((None, None, None)))) == 1
        formulaA.remove((None, RDF.type, None))
        assert len(list(formulaA.triples((None, None, None)))) == 0

        universe.remove((None, RDF.type, RDFS.Class))

        # remove_context tests
        universe.remove_context(formulaB)
        assert len(list(universe.triples((None, RDF.type, None)))) == 0
        assert len(universe) == 1
        assert len(formulaB) == 0

        universe.remove((None, None, None))
        assert len(universe) == 0

        g.close()
        if store == 'SQLite':
            os.unlink(path)
        else:
            g.store.destroy(configString)
    except:
        g.close()
        if store == 'SQLite':
            os.unlink(path)
        else:
            g.store.destroy(configString)
        raise


def testFormulaStores():
    pluginname = None
    if __name__ == '__main__':
        if len(sys.argv) > 1:
            pluginname = sys.argv[1]

    for s in plugin.plugins(pluginname, plugin.Store):
        if s.name in (
                'Auditable', 'Concurrent',
                'SPARQLStore', 'SPARQLUpdateStore',
        ):
            continue
        if not s.getClass().formula_aware:
            continue
        yield testFormulaStore, s.name


if __name__ == '__main__':
    import nose
    nose.main(defaultTest=sys.argv[0])



"""

RDFLib can map between data-typed literals and python objects.

Mapping for integers, floats, dateTimes, etc. are already added, but
you can also add your own.

This example shows how :meth:`rdflib.term.bind` lets you register new
mappings between literal datatypes and python objects

"""


from rdflib import Graph, Literal, Namespace, XSD
from rdflib.term import bind

if __name__ == '__main__':

    # complex numbers are not registered by default
    # no custom constructor/serializer needed since
    # complex('(2+3j)') works fine
    bind(XSD.complexNumber, complex)

    ns = Namespace("urn:my:namespace:")

    c = complex(2, 3)

    l = Literal(c)

    g = Graph()
    g.add((ns.mysubject, ns.myprop, l))

    n3 = g.serialize(format='n3')

    # round-trip through n3

    g2 = Graph()
    g2.parse(data=n3, format='n3')

    l2 = list(g2)[0][2]

    print(l2)

    print(l2.value == c)  # back to a python complex object



import rdflib

import nose.tools


def test_broken_add():

    g = rdflib.Graph()
    nose.tools.assert_raises(AssertionError, lambda: g.add((1, 2, 3)))
    nose.tools.assert_raises(AssertionError, lambda: g.addN([(1, 2, 3, g)]))


if __name__ == '__main__':
    import nose
    import sys
    nose.main(defaultTest=sys.argv[0])




from rdflib import ConjunctiveGraph, URIRef
import rdflib.plugin

from six import b


def testFinalNewline():
    """
    http://code.google.com/p/rdflib/issues/detail?id=5
    """
    import sys

    graph = ConjunctiveGraph()
    graph.add((URIRef("http://ex.org/a"),
               URIRef("http://ex.org/b"),
               URIRef("http://ex.org/c")))

    failed = set()
    for p in rdflib.plugin.plugins(None, rdflib.plugin.Serializer):
        v = graph.serialize(format=p.name)
        lines = v.split(b("\n"))
        if b("\n") not in v or (lines[-1] != b('')):
            failed.add(p.name)
    assert len(failed) == 0, "No final newline for formats: '%s'" % failed


if __name__ == "__main__":

    import sys
    import nose
    if len(sys.argv) == 1:
        nose.main(defaultTest=sys.argv[0])
